# coding=utf-8

# import json
import traceback
from flask_restful import Resource
from flask import current_app, request
from typing import Dict

from ...core.embedding import OpenAIEmbedding
from ...core.datasource import QDDataSource, OpenAIDataSource
from ...common import response, validator
from ...core.qa import QABot
from ...core.intent import IntentBot
from ...core.chatgpt import ChatGPTBot
from ...core import const

POST_SCHEMA = {
    "type": "object",
    "properties": {
        "prompt": {"type": "string"},
        "limit": {"type": "integer"},
        "score": {"type": "number"},
        "stream": {"type": "boolean"},
    },
    "required": ["prompt"],
}


class CompletionResource(Resource):
    def __init__(self):
        embedder = OpenAIEmbedding(current_app.config["OPENAI_API_KEY"])

        qd_ds = QDDataSource(
            current_app.config["QDRANT_HOST"],
            current_app.config["QDRANT_PORT"],
        )

        openai_ds = OpenAIDataSource(
            current_app.config["OPENAI_API_KEY"],
            float(current_app.config["OPENAI_TEMPERATURE"]),
        )

        qa_score = float(current_app.config["QA_SCORE"])
        qa_limit = int(current_app.config["QA_LIMIT"])
        qa_collection = current_app.config["QA_COLLECTION"]
        self.qa_bot = QABot(qd_ds, embedder, qa_collection, qa_score, qa_limit)

        intent_score = float(current_app.config["INTENT_SCORE"])
        intent_limit = int(current_app.config["INTENT_LIMIT"])
        intent_collection = current_app.config["INTENT_COLLECTION"]
        self.intent_bot = IntentBot(
            qd_ds, embedder, intent_collection, intent_score, intent_limit
        )

        self.chatgpt_bot = ChatGPTBot(openai_ds)

    @response.wrap_response
    @validator.validator(POST_SCHEMA)
    def post(self):
        data = request.get_json(force=True)
        req: Dict = data

        # if not req["stream"]:
        #     return self._completion(req)
        # else:
        #     pass



        # 1. intent check
        intent = self.intent_bot.complete(req["prompt"])

        # 1.1 intent ==> qa
        if intent is not None and intent.answer == const.INTENT_QA:
            try:
                answer = self.qa_bot.complete(**req)
                if answer is None:
                    return response.NotFound("No Answer Found", prompt=req["prompt"])
                return response.OK("OK", answer.to_dict())
            except Exception as e:
                traceback.print_exc()
                return response.InternalServerError(str(e))

        # 1.2 intent ==> chatgpt
        try:
            resp = self.chatgpt_bot.complete(req["prompt"])
            return response.OK("", resp.to_dict())
        except Exception as e:
            traceback.print_exc()
            return response.InternalServerError(str(e))

    def _completion(self, req: Dict):
        # 1. intent check
        intent = self.intent_bot.complete(req["prompt"])

        # 1.1 intent ==> qa
        if intent is not None and intent.answer == const.INTENT_QA:
            try:
                answer = self.qa_bot.complete(**req)
                if answer is None:
                    return response.NotFound("No Answer Found", prompt=req["prompt"])
                return response.OK("OK", answer.to_dict())
            except Exception as e:
                traceback.print_exc()
                return response.InternalServerError(str(e))

        # 1.2 intent ==> chatgpt
        try:
            resp = self.chatgpt_bot.complete(req["prompt"])
            return response.OK("", resp.to_dict())
        except Exception as e:
            traceback.print_exc()
            return response.InternalServerError(str(e))


    def _stream_completion(self, req):
        # # 1. intent check
        # intent = self.intent_bot.complete(req["prompt"])
        pass
