# coding=utf-8

import csv
import openai
from abc import abstractmethod, ABCMeta
from typing import List, Callable, Dict, Union

from qdrant_client import QdrantClient
from qdrant_client.http.models import Distance, VectorParams
from qdrant_client.http.models import PointStruct

from .embedding import Embedding

# from ..util import util


class DataSource(metaclass=ABCMeta):
    @abstractmethod
    def search(
        self, collection: str, prompt: str, limit: int, **kwargs
    ) -> Union[List, str]:
        pass


# @util.singleton
class QDDataSource:
    def __init__(self, host: str, port: int):
        self.client = QdrantClient(host=host, port=port)

    def create_collection(
        self, collection: str, size: int = 1536, distance: Distance = Distance.COSINE
    ):
        """创建数据集

        Args:
            collection (str): 集合名称
            size (int): 向量大小
            distance (QdrantClient.Distance): 损失函数
        """
        try:
            self.client.recreate_collection(
                collection_name=collection,
                vectors_config=VectorParams(size=size, distance=distance),
            )
        except Exception as e:
            raise e

    def upload_csv(
        self,
        collection: str,
        embedder: Embedding,
        filepath: str,
        formatter: Callable[[List], Dict],
    ):
        """导入csv数据集

        Args:
            filepath: 文件路径
        """
        with open(filepath, "r", encoding="utf-8") as f:
            csv_reader = csv.reader(f)
            count = 0
            for line in csv_reader:
                vec = embedder.get_vector(line[0])
                print(line, vec)
                self.client.upsert(
                    collection_name=collection,
                    wait=True,
                    points=[
                        PointStruct(
                            id=count,
                            vector=vec,
                            payload=formatter(line),
                        )
                    ],
                )
                count += 1

    def search(
        self,
        prompt: str,
        limit: int,
        embedder: Embedding = None,
        collection: str = "",
    ) -> List:
        if embedder is None:
            raise ValueError("Embedding Object is None")

        if collection is None or len(collection) == 0:
            raise ValueError("Collection is None")

        vec = embedder.get_vector(prompt)
        results = self.client.search(
            collection_name=collection,
            query_vector=vec,
            limit=limit,
            search_params={"exact": False, "hnsw_ef": 128},
        )
        return results


class OpenAIDataSource:
    def __init__(self, api_key: str, temperature: float):
        self.api_key = api_key
        self.temperature = temperature
        openai.api_key = api_key

    def search(self, prompt: str, limit: int = 0, model="gpt-3.5-turbo") -> str:
        completion = openai.ChatCompletion.create(
            temperature=self.temperature,
            model=model,
            messages=[{"role": "user", "content": prompt}],
        )

        return completion.choices[0].message.content
